// Copyright (c) 2011 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#include <GLES2/gl2.h>
#include <stdio.h>
#include <unistd.h>


#include "ppapi_njf_demo.h"

int16_t TriangleWave(int64_t counter, int32_t freq, float amplitude) {
  int32_t invfreq = 48000 / freq;
  float range = 4.0f * (counter % invfreq) / float(invfreq);
  float toint16 = 32767.0f;
  float wave;
  if (range > 3.0f)
    wave = range - 4.0f;
  else if (range > 1.0f)
    wave = 2.0f - range;
  else wave = range;
  wave = wave * amplitude * toint16;
  if (wave > toint16)
    wave = toint16;
  else if (wave < -toint16)
    wave = -toint16;
  return int16_t(wave);
}


#include <stdint.h>
 
extern "C" {
  __inline__ uint64_t RDTSC(void) {
    uint32_t lo, hi;
    __asm__ __volatile__ (      // serialize
    "xorl %%eax,%%eax \n        cpuid"
    ::: "%rax", "%rbx", "%rcx", "%rdx");
    /* We cannot use "=A", since this would use %rax on x86_64 and return only the lower 32bits of the TSC */
    __asm__ __volatile__ ("rdtsc" : "=a" (lo), "=d" (hi));
    return (uint64_t)hi << 32 | lo;
  }
}

void UpdateTimer(InstanceInfo* info) {
  if (info->clock_warmup < 20) {
    info->clock_warmup++;
    return;
  }
  if (info->clock_previous == 0) {
    info->clock_previous = RDTSC();
    return;
  }

  int64_t this_clock = RDTSC();
  int64_t delta = this_clock - info->clock_previous;
  // if (delta > 0)
  {
    double cpf_a = (((double)info->clock_per_frame_a * 9) + delta) /
                  (10);
    info->clock_per_frame_a = (int64_t)cpf_a;
    double cpf_b = (((double)info->clock_per_frame_b * 9) + cpf_a) /
                  (10);
    info->clock_per_frame_b = (int64_t)cpf_b;
    double cpf_c = (((double)info->clock_per_frame_c * 9) + cpf_b) /
                  (10);
    info->clock_per_frame_c = (int64_t)cpf_c;

    info->clock_freq = (info->sample_rate * info->clock_per_frame_c) /
                       (info->sample_frame_count);

    info->clock_delta = delta;
    info->clock_previous = this_clock;
    info->clock_count++;
  }
}

void* clock_thread(void* data) {
  int64_t clock_previous = 0;
  while(1) {
    usleep(10000);
    int64_t this_clock = RDTSC();
    int64_t delta = this_clock - clock_previous;
    printf("delta: %12.2f\n", delta * 1.0);
    clock_previous = this_clock;
  }
  return NULL;
}

void AudioCallback(void* sample_buffer,
                   uint32_t buffer_size_in_bytes,
                   void* user_data) {
  InstanceInfo* info = static_cast<InstanceInfo*>(user_data);
  int16_t* stereo16 = static_cast<int16_t*>(sample_buffer);
  static int16_t sample = 0;
  static int64_t counter = 0;
  // UpdateTimer(info);
  for (int32_t i = 0; i < info->sample_frame_count; i++) {
    if (info->keys_pressed)
      sample = TriangleWave(counter, 440, 0.5f);
    stereo16[i * 2 + 0] = sample;  // Left channel
    stereo16[i * 2 + 1] = sample;  // Right channel
    counter++;
  }
}

